""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class PartZonelets(CoreObject):
    """A structure containing some or all face zonelet ids available in a part.
    """
    _default_params = {}

    def __initialize(
            self,
            part_id: int,
            face_zonelets: Iterable[int]):
        self._part_id = part_id
        self._face_zonelets = face_zonelets if isinstance(face_zonelets, np.ndarray) else np.array(face_zonelets, dtype=np.int32) if face_zonelets is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            part_id: int = None,
            face_zonelets: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the PartZonelets.

        Parameters
        ----------
        model: Model
            Model to create a PartZonelets object with default parameters.
        part_id: int, optional
            Id of part.
        face_zonelets: Iterable[int], optional
            List of face zonelet ids available in the part.
        json_data: dict, optional
            JSON dictionary to create a PartZonelets object with provided parameters.

        Examples
        --------
        >>> part_zonelets = prime.PartZonelets(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["partID"] if "partID" in json_data else None,
                json_data["faceZonelets"] if "faceZonelets" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [part_id, face_zonelets])
            if all_field_specified:
                self.__initialize(
                    part_id,
                    face_zonelets)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "PartZonelets")
                    json_data = param_json["PartZonelets"] if "PartZonelets" in param_json else {}
                    self.__initialize(
                        part_id if part_id is not None else ( PartZonelets._default_params["part_id"] if "part_id" in PartZonelets._default_params else (json_data["partID"] if "partID" in json_data else None)),
                        face_zonelets if face_zonelets is not None else ( PartZonelets._default_params["face_zonelets"] if "face_zonelets" in PartZonelets._default_params else (json_data["faceZonelets"] if "faceZonelets" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            part_id: int = None,
            face_zonelets: Iterable[int] = None):
        """Set the default values of PartZonelets.

        Parameters
        ----------
        part_id: int, optional
            Id of part.
        face_zonelets: Iterable[int], optional
            List of face zonelet ids available in the part.
        """
        args = locals()
        [PartZonelets._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of PartZonelets.

        Examples
        --------
        >>> PartZonelets.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in PartZonelets._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._part_id is not None:
            json_data["partID"] = self._part_id
        if self._face_zonelets is not None:
            json_data["faceZonelets"] = self._face_zonelets
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "part_id :  %s\nface_zonelets :  %s" % (self._part_id, self._face_zonelets)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def part_id(self) -> int:
        """Id of part.
        """
        return self._part_id

    @part_id.setter
    def part_id(self, value: int):
        self._part_id = value

    @property
    def face_zonelets(self) -> Iterable[int]:
        """List of face zonelet ids available in the part.
        """
        return self._face_zonelets

    @face_zonelets.setter
    def face_zonelets(self, value: Iterable[int]):
        self._face_zonelets = value

class SetNameResults(CoreObject):
    """Results associated with the set name.
    """
    _default_params = {}

    def __initialize(
            self,
            warning_code: WarningCode,
            assigned_name: str,
            error_code: ErrorCode):
        self._warning_code = WarningCode(warning_code)
        self._assigned_name = assigned_name
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            warning_code: WarningCode = None,
            assigned_name: str = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SetNameResults.

        Parameters
        ----------
        model: Model
            Model to create a SetNameResults object with default parameters.
        warning_code: WarningCode, optional
            Warning code associated with the set name of given entity.
        assigned_name: str, optional
            Assigned name of given entity.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        json_data: dict, optional
            JSON dictionary to create a SetNameResults object with provided parameters.

        Examples
        --------
        >>> set_name_results = prime.SetNameResults(model = model)
        """
        if json_data:
            self.__initialize(
                WarningCode(json_data["warningCode"] if "warningCode" in json_data else None),
                json_data["assignedName"] if "assignedName" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [warning_code, assigned_name, error_code])
            if all_field_specified:
                self.__initialize(
                    warning_code,
                    assigned_name,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SetNameResults")
                    json_data = param_json["SetNameResults"] if "SetNameResults" in param_json else {}
                    self.__initialize(
                        warning_code if warning_code is not None else ( SetNameResults._default_params["warning_code"] if "warning_code" in SetNameResults._default_params else WarningCode(json_data["warningCode"] if "warningCode" in json_data else None)),
                        assigned_name if assigned_name is not None else ( SetNameResults._default_params["assigned_name"] if "assigned_name" in SetNameResults._default_params else (json_data["assignedName"] if "assignedName" in json_data else None)),
                        error_code if error_code is not None else ( SetNameResults._default_params["error_code"] if "error_code" in SetNameResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            warning_code: WarningCode = None,
            assigned_name: str = None,
            error_code: ErrorCode = None):
        """Set the default values of SetNameResults.

        Parameters
        ----------
        warning_code: WarningCode, optional
            Warning code associated with the set name of given entity.
        assigned_name: str, optional
            Assigned name of given entity.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        """
        args = locals()
        [SetNameResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SetNameResults.

        Examples
        --------
        >>> SetNameResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SetNameResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._warning_code is not None:
            json_data["warningCode"] = self._warning_code
        if self._assigned_name is not None:
            json_data["assignedName"] = self._assigned_name
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "warning_code :  %s\nassigned_name :  %s\nerror_code :  %s" % (self._warning_code, self._assigned_name, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def warning_code(self) -> WarningCode:
        """Warning code associated with the set name of given entity.
        """
        return self._warning_code

    @warning_code.setter
    def warning_code(self, value: WarningCode):
        self._warning_code = value

    @property
    def assigned_name(self) -> str:
        """Assigned name of given entity.
        """
        return self._assigned_name

    @assigned_name.setter
    def assigned_name(self, value: str):
        self._assigned_name = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class DeleteResults(CoreObject):
    """Results associated with the deletion of items.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode):
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the DeleteResults.

        Parameters
        ----------
        model: Model
            Model to create a DeleteResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        json_data: dict, optional
            JSON dictionary to create a DeleteResults object with provided parameters.

        Examples
        --------
        >>> delete_results = prime.DeleteResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [error_code])
            if all_field_specified:
                self.__initialize(
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "DeleteResults")
                    json_data = param_json["DeleteResults"] if "DeleteResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( DeleteResults._default_params["error_code"] if "error_code" in DeleteResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None):
        """Set the default values of DeleteResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        """
        args = locals()
        [DeleteResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of DeleteResults.

        Examples
        --------
        >>> DeleteResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in DeleteResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s" % (self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class CopyZoneletsParams(CoreObject):
    """Parameters to copy zonelets.
    """
    _default_params = {}

    def __initialize(
            self,
            copy_zones: bool):
        self._copy_zones = copy_zones

    def __init__(
            self,
            model: CommunicationManager=None,
            copy_zones: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CopyZoneletsParams.

        Parameters
        ----------
        model: Model
            Model to create a CopyZoneletsParams object with default parameters.
        copy_zones: bool, optional
            Option to copy zones of input zonelets to corresponding copied zonelets.
        json_data: dict, optional
            JSON dictionary to create a CopyZoneletsParams object with provided parameters.

        Examples
        --------
        >>> copy_zonelets_params = prime.CopyZoneletsParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["copyZones"] if "copyZones" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [copy_zones])
            if all_field_specified:
                self.__initialize(
                    copy_zones)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CopyZoneletsParams")
                    json_data = param_json["CopyZoneletsParams"] if "CopyZoneletsParams" in param_json else {}
                    self.__initialize(
                        copy_zones if copy_zones is not None else ( CopyZoneletsParams._default_params["copy_zones"] if "copy_zones" in CopyZoneletsParams._default_params else (json_data["copyZones"] if "copyZones" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            copy_zones: bool = None):
        """Set the default values of CopyZoneletsParams.

        Parameters
        ----------
        copy_zones: bool, optional
            Option to copy zones of input zonelets to corresponding copied zonelets.
        """
        args = locals()
        [CopyZoneletsParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CopyZoneletsParams.

        Examples
        --------
        >>> CopyZoneletsParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CopyZoneletsParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._copy_zones is not None:
            json_data["copyZones"] = self._copy_zones
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "copy_zones :  %s" % (self._copy_zones)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def copy_zones(self) -> bool:
        """Option to copy zones of input zonelets to corresponding copied zonelets.
        """
        return self._copy_zones

    @copy_zones.setter
    def copy_zones(self, value: bool):
        self._copy_zones = value

class CopyZoneletsResults(CoreObject):
    """Result structure associated with copying zonelets.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            copied_zonelets: Iterable[int],
            copied_face_zonelets: Iterable[int]):
        self._error_code = ErrorCode(error_code)
        self._copied_zonelets = copied_zonelets if isinstance(copied_zonelets, np.ndarray) else np.array(copied_zonelets, dtype=np.int32) if copied_zonelets is not None else None
        self._copied_face_zonelets = copied_face_zonelets if isinstance(copied_face_zonelets, np.ndarray) else np.array(copied_face_zonelets, dtype=np.int32) if copied_face_zonelets is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            copied_zonelets: Iterable[int] = None,
            copied_face_zonelets: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CopyZoneletsResults.

        Parameters
        ----------
        model: Model
            Model to create a CopyZoneletsResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with failure of operation.
        copied_zonelets: Iterable[int], optional
            Ids of the copied zonelets.
        copied_face_zonelets: Iterable[int], optional
            Ids of the copied bounding faces of cell zonelets.
        json_data: dict, optional
            JSON dictionary to create a CopyZoneletsResults object with provided parameters.

        Examples
        --------
        >>> copy_zonelets_results = prime.CopyZoneletsResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["copiedZonelets"] if "copiedZonelets" in json_data else None,
                json_data["copiedFaceZonelets"] if "copiedFaceZonelets" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, copied_zonelets, copied_face_zonelets])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    copied_zonelets,
                    copied_face_zonelets)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CopyZoneletsResults")
                    json_data = param_json["CopyZoneletsResults"] if "CopyZoneletsResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( CopyZoneletsResults._default_params["error_code"] if "error_code" in CopyZoneletsResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        copied_zonelets if copied_zonelets is not None else ( CopyZoneletsResults._default_params["copied_zonelets"] if "copied_zonelets" in CopyZoneletsResults._default_params else (json_data["copiedZonelets"] if "copiedZonelets" in json_data else None)),
                        copied_face_zonelets if copied_face_zonelets is not None else ( CopyZoneletsResults._default_params["copied_face_zonelets"] if "copied_face_zonelets" in CopyZoneletsResults._default_params else (json_data["copiedFaceZonelets"] if "copiedFaceZonelets" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            copied_zonelets: Iterable[int] = None,
            copied_face_zonelets: Iterable[int] = None):
        """Set the default values of CopyZoneletsResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with failure of operation.
        copied_zonelets: Iterable[int], optional
            Ids of the copied zonelets.
        copied_face_zonelets: Iterable[int], optional
            Ids of the copied bounding faces of cell zonelets.
        """
        args = locals()
        [CopyZoneletsResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CopyZoneletsResults.

        Examples
        --------
        >>> CopyZoneletsResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CopyZoneletsResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._copied_zonelets is not None:
            json_data["copiedZonelets"] = self._copied_zonelets
        if self._copied_face_zonelets is not None:
            json_data["copiedFaceZonelets"] = self._copied_face_zonelets
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\ncopied_zonelets :  %s\ncopied_face_zonelets :  %s" % (self._error_code, self._copied_zonelets, self._copied_face_zonelets)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def copied_zonelets(self) -> Iterable[int]:
        """Ids of the copied zonelets.
        """
        return self._copied_zonelets

    @copied_zonelets.setter
    def copied_zonelets(self, value: Iterable[int]):
        self._copied_zonelets = value

    @property
    def copied_face_zonelets(self) -> Iterable[int]:
        """Ids of the copied bounding faces of cell zonelets.
        """
        return self._copied_face_zonelets

    @copied_face_zonelets.setter
    def copied_face_zonelets(self, value: Iterable[int]):
        self._copied_face_zonelets = value



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class JoinSeparateMethod(enum.IntEnum):
    """Join separation method to handle separated overlapped zonelets.
    """
    KEEPNONE = 0
    """Remove both overlapping zonelets after joining."""
    KEEPONE = 1
    """Keep one overlap zonelets after joining."""
    KEEPBOTH = 2
    """Keep both overlapping zonelets after joining."""

class StitchType(enum.IntEnum):
    """Stitch type depending on nature of surface boundary edges to be stitched.
    """
    ALLALL = 0
    """Stitch surfaces at boundary edges."""
    FREEFREE = 1
    """Stitch surfaces at free boundary edges."""

class MatchedMeshOption(enum.IntEnum):
    """Type to specify treatment of matched mesh.
    """
    NONE = 0
    """No action to be taken for matched mesh."""
    TRIMONESIDE = 3
    """Delete matched faces on one side and merge matched nodes at middle locations (works only within a single part)."""
    TRIMTWOSIDES = 4
    """Delete matched faces on both sides and merge matched nodes at middle locations (works only within a single part)."""

class OverlapPairs(CoreObject):
    """Provides ids of a pair of overlapping face zonelets.
    """
    _default_params = {}

    def __initialize(
            self,
            zone_id0: int,
            zone_id1: int):
        self._zone_id0 = zone_id0
        self._zone_id1 = zone_id1

    def __init__(
            self,
            model: CommunicationManager=None,
            zone_id0: int = None,
            zone_id1: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the OverlapPairs.

        Parameters
        ----------
        model: Model
            Model to create a OverlapPairs object with default parameters.
        zone_id0: int, optional
            Id of one overlapping face zonelet.
        zone_id1: int, optional
            Id of other overlapping face zonelet.
        json_data: dict, optional
            JSON dictionary to create a OverlapPairs object with provided parameters.

        Examples
        --------
        >>> overlap_pairs = prime.OverlapPairs(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["zoneId0"] if "zoneId0" in json_data else None,
                json_data["zoneId1"] if "zoneId1" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [zone_id0, zone_id1])
            if all_field_specified:
                self.__initialize(
                    zone_id0,
                    zone_id1)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "OverlapPairs")
                    json_data = param_json["OverlapPairs"] if "OverlapPairs" in param_json else {}
                    self.__initialize(
                        zone_id0 if zone_id0 is not None else ( OverlapPairs._default_params["zone_id0"] if "zone_id0" in OverlapPairs._default_params else (json_data["zoneId0"] if "zoneId0" in json_data else None)),
                        zone_id1 if zone_id1 is not None else ( OverlapPairs._default_params["zone_id1"] if "zone_id1" in OverlapPairs._default_params else (json_data["zoneId1"] if "zoneId1" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            zone_id0: int = None,
            zone_id1: int = None):
        """Set the default values of OverlapPairs.

        Parameters
        ----------
        zone_id0: int, optional
            Id of one overlapping face zonelet.
        zone_id1: int, optional
            Id of other overlapping face zonelet.
        """
        args = locals()
        [OverlapPairs._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of OverlapPairs.

        Examples
        --------
        >>> OverlapPairs.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in OverlapPairs._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._zone_id0 is not None:
            json_data["zoneId0"] = self._zone_id0
        if self._zone_id1 is not None:
            json_data["zoneId1"] = self._zone_id1
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "zone_id0 :  %s\nzone_id1 :  %s" % (self._zone_id0, self._zone_id1)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def zone_id0(self) -> int:
        """Id of one overlapping face zonelet.
        """
        return self._zone_id0

    @zone_id0.setter
    def zone_id0(self, value: int):
        self._zone_id0 = value

    @property
    def zone_id1(self) -> int:
        """Id of other overlapping face zonelet.
        """
        return self._zone_id1

    @zone_id1.setter
    def zone_id1(self, value: int):
        self._zone_id1 = value

class OverlapSearchResults(CoreObject):
    """Provides ids of a pair of overlapping face zonelets.
    """
    _default_params = {}

    def __initialize(
            self,
            n_pairs: int,
            overlap_pairs: List[OverlapPairs],
            error_code: ErrorCode):
        self._n_pairs = n_pairs
        self._overlap_pairs = overlap_pairs
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_pairs: int = None,
            overlap_pairs: List[OverlapPairs] = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the OverlapSearchResults.

        Parameters
        ----------
        model: Model
            Model to create a OverlapSearchResults object with default parameters.
        n_pairs: int, optional
            Number of pairs.
        overlap_pairs: List[OverlapPairs], optional
            Ids corresponding to pairs of overlapping face zonelets.
        error_code: ErrorCode, optional
            Error Code associated with failure of operation.
        json_data: dict, optional
            JSON dictionary to create a OverlapSearchResults object with provided parameters.

        Examples
        --------
        >>> overlap_search_results = prime.OverlapSearchResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nPairs"] if "nPairs" in json_data else None,
                [OverlapPairs(model = model, json_data = data) for data in json_data["overlapPairs"]] if "overlapPairs" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_pairs, overlap_pairs, error_code])
            if all_field_specified:
                self.__initialize(
                    n_pairs,
                    overlap_pairs,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "OverlapSearchResults")
                    json_data = param_json["OverlapSearchResults"] if "OverlapSearchResults" in param_json else {}
                    self.__initialize(
                        n_pairs if n_pairs is not None else ( OverlapSearchResults._default_params["n_pairs"] if "n_pairs" in OverlapSearchResults._default_params else (json_data["nPairs"] if "nPairs" in json_data else None)),
                        overlap_pairs if overlap_pairs is not None else ( OverlapSearchResults._default_params["overlap_pairs"] if "overlap_pairs" in OverlapSearchResults._default_params else [OverlapPairs(model = model, json_data = data) for data in (json_data["overlapPairs"] if "overlapPairs" in json_data else None)]),
                        error_code if error_code is not None else ( OverlapSearchResults._default_params["error_code"] if "error_code" in OverlapSearchResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_pairs: int = None,
            overlap_pairs: List[OverlapPairs] = None,
            error_code: ErrorCode = None):
        """Set the default values of OverlapSearchResults.

        Parameters
        ----------
        n_pairs: int, optional
            Number of pairs.
        overlap_pairs: List[OverlapPairs], optional
            Ids corresponding to pairs of overlapping face zonelets.
        error_code: ErrorCode, optional
            Error Code associated with failure of operation.
        """
        args = locals()
        [OverlapSearchResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of OverlapSearchResults.

        Examples
        --------
        >>> OverlapSearchResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in OverlapSearchResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_pairs is not None:
            json_data["nPairs"] = self._n_pairs
        if self._overlap_pairs is not None:
            json_data["overlapPairs"] = [data._jsonify() for data in self._overlap_pairs]
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_pairs :  %s\noverlap_pairs :  %s\nerror_code :  %s" % (self._n_pairs, '[' + ''.join('\n' + str(data) for data in self._overlap_pairs) + ']', self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_pairs(self) -> int:
        """Number of pairs.
        """
        return self._n_pairs

    @n_pairs.setter
    def n_pairs(self, value: int):
        self._n_pairs = value

    @property
    def overlap_pairs(self) -> List[OverlapPairs]:
        """Ids corresponding to pairs of overlapping face zonelets.
        """
        return self._overlap_pairs

    @overlap_pairs.setter
    def overlap_pairs(self, value: List[OverlapPairs]):
        self._overlap_pairs = value

    @property
    def error_code(self) -> ErrorCode:
        """Error Code associated with failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class ConnectResults(CoreObject):
    """Results associated with the connection operations.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            warning_codes: List[WarningCode]):
        self._error_code = ErrorCode(error_code)
        self._warning_codes = warning_codes

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ConnectResults.

        Parameters
        ----------
        model: Model
            Model to create a ConnectResults object with default parameters.
        error_code: ErrorCode, optional
            Error Code associated with failure of operation.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the operation.
        json_data: dict, optional
            JSON dictionary to create a ConnectResults object with provided parameters.

        Examples
        --------
        >>> connect_results = prime.ConnectResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                [WarningCode(data) for data in json_data["warningCodes"]] if "warningCodes" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, warning_codes])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    warning_codes)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ConnectResults")
                    json_data = param_json["ConnectResults"] if "ConnectResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( ConnectResults._default_params["error_code"] if "error_code" in ConnectResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        warning_codes if warning_codes is not None else ( ConnectResults._default_params["warning_codes"] if "warning_codes" in ConnectResults._default_params else [WarningCode(data) for data in (json_data["warningCodes"] if "warningCodes" in json_data else None)]))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None):
        """Set the default values of ConnectResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error Code associated with failure of operation.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the operation.
        """
        args = locals()
        [ConnectResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ConnectResults.

        Examples
        --------
        >>> ConnectResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ConnectResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._warning_codes is not None:
            json_data["warningCodes"] = [data for data in self._warning_codes]
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nwarning_codes :  %s" % (self._error_code, '[' + ''.join('\n' + str(data) for data in self._warning_codes) + ']')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error Code associated with failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def warning_codes(self) -> List[WarningCode]:
        """Warning codes associated with the operation.
        """
        return self._warning_codes

    @warning_codes.setter
    def warning_codes(self, value: List[WarningCode]):
        self._warning_codes = value

class IntersectParams(CoreObject):
    """Parameters used for intersection.
    """
    _default_params = {}

    def __initialize(
            self,
            tolerance: float,
            use_absolute_tolerance: bool,
            remesh: bool):
        self._tolerance = tolerance
        self._use_absolute_tolerance = use_absolute_tolerance
        self._remesh = remesh

    def __init__(
            self,
            model: CommunicationManager=None,
            tolerance: float = None,
            use_absolute_tolerance: bool = None,
            remesh: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the IntersectParams.

        Parameters
        ----------
        model: Model
            Model to create a IntersectParams object with default parameters.
        tolerance: float, optional
            Intersection tolerance.
        use_absolute_tolerance: bool, optional
            True if tolerance provided is absolute value.
        remesh: bool, optional
            Local remesh at the intersection.
        json_data: dict, optional
            JSON dictionary to create a IntersectParams object with provided parameters.

        Examples
        --------
        >>> intersect_params = prime.IntersectParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["tolerance"] if "tolerance" in json_data else None,
                json_data["useAbsoluteTolerance"] if "useAbsoluteTolerance" in json_data else None,
                json_data["remesh"] if "remesh" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [tolerance, use_absolute_tolerance, remesh])
            if all_field_specified:
                self.__initialize(
                    tolerance,
                    use_absolute_tolerance,
                    remesh)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "IntersectParams")
                    json_data = param_json["IntersectParams"] if "IntersectParams" in param_json else {}
                    self.__initialize(
                        tolerance if tolerance is not None else ( IntersectParams._default_params["tolerance"] if "tolerance" in IntersectParams._default_params else (json_data["tolerance"] if "tolerance" in json_data else None)),
                        use_absolute_tolerance if use_absolute_tolerance is not None else ( IntersectParams._default_params["use_absolute_tolerance"] if "use_absolute_tolerance" in IntersectParams._default_params else (json_data["useAbsoluteTolerance"] if "useAbsoluteTolerance" in json_data else None)),
                        remesh if remesh is not None else ( IntersectParams._default_params["remesh"] if "remesh" in IntersectParams._default_params else (json_data["remesh"] if "remesh" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            tolerance: float = None,
            use_absolute_tolerance: bool = None,
            remesh: bool = None):
        """Set the default values of IntersectParams.

        Parameters
        ----------
        tolerance: float, optional
            Intersection tolerance.
        use_absolute_tolerance: bool, optional
            True if tolerance provided is absolute value.
        remesh: bool, optional
            Local remesh at the intersection.
        """
        args = locals()
        [IntersectParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of IntersectParams.

        Examples
        --------
        >>> IntersectParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in IntersectParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._tolerance is not None:
            json_data["tolerance"] = self._tolerance
        if self._use_absolute_tolerance is not None:
            json_data["useAbsoluteTolerance"] = self._use_absolute_tolerance
        if self._remesh is not None:
            json_data["remesh"] = self._remesh
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "tolerance :  %s\nuse_absolute_tolerance :  %s\nremesh :  %s" % (self._tolerance, self._use_absolute_tolerance, self._remesh)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def tolerance(self) -> float:
        """Intersection tolerance.
        """
        return self._tolerance

    @tolerance.setter
    def tolerance(self, value: float):
        self._tolerance = value

    @property
    def use_absolute_tolerance(self) -> bool:
        """True if tolerance provided is absolute value.
        """
        return self._use_absolute_tolerance

    @use_absolute_tolerance.setter
    def use_absolute_tolerance(self, value: bool):
        self._use_absolute_tolerance = value

    @property
    def remesh(self) -> bool:
        """Local remesh at the intersection.
        """
        return self._remesh

    @remesh.setter
    def remesh(self, value: bool):
        self._remesh = value

class JoinParams(CoreObject):
    """Parameters used for join.
    """
    _default_params = {}

    def __initialize(
            self,
            tolerance: float,
            use_absolute_tolerance: bool,
            remesh: bool,
            match_angle: float,
            overlap_zone_name: str):
        self._tolerance = tolerance
        self._use_absolute_tolerance = use_absolute_tolerance
        self._remesh = remesh
        self._match_angle = match_angle
        self._overlap_zone_name = overlap_zone_name

    def __init__(
            self,
            model: CommunicationManager=None,
            tolerance: float = None,
            use_absolute_tolerance: bool = None,
            remesh: bool = None,
            match_angle: float = None,
            overlap_zone_name: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the JoinParams.

        Parameters
        ----------
        model: Model
            Model to create a JoinParams object with default parameters.
        tolerance: float, optional
            Overlap tolerance between overlapping zonelets.
        use_absolute_tolerance: bool, optional
            Tolerance provided is absolute value.
        remesh: bool, optional
            Remesh at overlap surface boundary.
        match_angle: float, optional
            Match angle determines face pair inclination for overlap consideration.
        overlap_zone_name: str, optional
            Zone id to be assigned to overlap zonelets belonging to different zones.
        json_data: dict, optional
            JSON dictionary to create a JoinParams object with provided parameters.

        Examples
        --------
        >>> join_params = prime.JoinParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["tolerance"] if "tolerance" in json_data else None,
                json_data["useAbsoluteTolerance"] if "useAbsoluteTolerance" in json_data else None,
                json_data["remesh"] if "remesh" in json_data else None,
                json_data["matchAngle"] if "matchAngle" in json_data else None,
                json_data["overlapZoneName"] if "overlapZoneName" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [tolerance, use_absolute_tolerance, remesh, match_angle, overlap_zone_name])
            if all_field_specified:
                self.__initialize(
                    tolerance,
                    use_absolute_tolerance,
                    remesh,
                    match_angle,
                    overlap_zone_name)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "JoinParams")
                    json_data = param_json["JoinParams"] if "JoinParams" in param_json else {}
                    self.__initialize(
                        tolerance if tolerance is not None else ( JoinParams._default_params["tolerance"] if "tolerance" in JoinParams._default_params else (json_data["tolerance"] if "tolerance" in json_data else None)),
                        use_absolute_tolerance if use_absolute_tolerance is not None else ( JoinParams._default_params["use_absolute_tolerance"] if "use_absolute_tolerance" in JoinParams._default_params else (json_data["useAbsoluteTolerance"] if "useAbsoluteTolerance" in json_data else None)),
                        remesh if remesh is not None else ( JoinParams._default_params["remesh"] if "remesh" in JoinParams._default_params else (json_data["remesh"] if "remesh" in json_data else None)),
                        match_angle if match_angle is not None else ( JoinParams._default_params["match_angle"] if "match_angle" in JoinParams._default_params else (json_data["matchAngle"] if "matchAngle" in json_data else None)),
                        overlap_zone_name if overlap_zone_name is not None else ( JoinParams._default_params["overlap_zone_name"] if "overlap_zone_name" in JoinParams._default_params else (json_data["overlapZoneName"] if "overlapZoneName" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            tolerance: float = None,
            use_absolute_tolerance: bool = None,
            remesh: bool = None,
            match_angle: float = None,
            overlap_zone_name: str = None):
        """Set the default values of JoinParams.

        Parameters
        ----------
        tolerance: float, optional
            Overlap tolerance between overlapping zonelets.
        use_absolute_tolerance: bool, optional
            Tolerance provided is absolute value.
        remesh: bool, optional
            Remesh at overlap surface boundary.
        match_angle: float, optional
            Match angle determines face pair inclination for overlap consideration.
        overlap_zone_name: str, optional
            Zone id to be assigned to overlap zonelets belonging to different zones.
        """
        args = locals()
        [JoinParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of JoinParams.

        Examples
        --------
        >>> JoinParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in JoinParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._tolerance is not None:
            json_data["tolerance"] = self._tolerance
        if self._use_absolute_tolerance is not None:
            json_data["useAbsoluteTolerance"] = self._use_absolute_tolerance
        if self._remesh is not None:
            json_data["remesh"] = self._remesh
        if self._match_angle is not None:
            json_data["matchAngle"] = self._match_angle
        if self._overlap_zone_name is not None:
            json_data["overlapZoneName"] = self._overlap_zone_name
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "tolerance :  %s\nuse_absolute_tolerance :  %s\nremesh :  %s\nmatch_angle :  %s\noverlap_zone_name :  %s" % (self._tolerance, self._use_absolute_tolerance, self._remesh, self._match_angle, self._overlap_zone_name)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def tolerance(self) -> float:
        """Overlap tolerance between overlapping zonelets.
        """
        return self._tolerance

    @tolerance.setter
    def tolerance(self, value: float):
        self._tolerance = value

    @property
    def use_absolute_tolerance(self) -> bool:
        """Tolerance provided is absolute value.
        """
        return self._use_absolute_tolerance

    @use_absolute_tolerance.setter
    def use_absolute_tolerance(self, value: bool):
        self._use_absolute_tolerance = value

    @property
    def remesh(self) -> bool:
        """Remesh at overlap surface boundary.
        """
        return self._remesh

    @remesh.setter
    def remesh(self, value: bool):
        self._remesh = value

    @property
    def match_angle(self) -> float:
        """Match angle determines face pair inclination for overlap consideration.
        """
        return self._match_angle

    @match_angle.setter
    def match_angle(self, value: float):
        self._match_angle = value

    @property
    def overlap_zone_name(self) -> str:
        """Zone id to be assigned to overlap zonelets belonging to different zones.
        """
        return self._overlap_zone_name

    @overlap_zone_name.setter
    def overlap_zone_name(self, value: str):
        self._overlap_zone_name = value

class SubtractVolumesParams(CoreObject):
    """Parameters to control the volume subtract operation.
    """
    _default_params = {}

    def __initialize(
            self,
            ignore_face_zonelets: Iterable[int],
            check_cutters: bool):
        self._ignore_face_zonelets = ignore_face_zonelets if isinstance(ignore_face_zonelets, np.ndarray) else np.array(ignore_face_zonelets, dtype=np.int32) if ignore_face_zonelets is not None else None
        self._check_cutters = check_cutters

    def __init__(
            self,
            model: CommunicationManager=None,
            ignore_face_zonelets: Iterable[int] = None,
            check_cutters: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SubtractVolumesParams.

        Parameters
        ----------
        model: Model
            Model to create a SubtractVolumesParams object with default parameters.
        ignore_face_zonelets: Iterable[int], optional
            Face zonelet ids that subtract volumes should not remove.(For example, periodic or fluid cap zonelets)
        check_cutters: bool, optional
            Option to handle overlapping or intersecting cutter volumes.
        json_data: dict, optional
            JSON dictionary to create a SubtractVolumesParams object with provided parameters.

        Examples
        --------
        >>> subtract_volumes_params = prime.SubtractVolumesParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["ignoreFaceZonelets"] if "ignoreFaceZonelets" in json_data else None,
                json_data["checkCutters"] if "checkCutters" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [ignore_face_zonelets, check_cutters])
            if all_field_specified:
                self.__initialize(
                    ignore_face_zonelets,
                    check_cutters)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SubtractVolumesParams")
                    json_data = param_json["SubtractVolumesParams"] if "SubtractVolumesParams" in param_json else {}
                    self.__initialize(
                        ignore_face_zonelets if ignore_face_zonelets is not None else ( SubtractVolumesParams._default_params["ignore_face_zonelets"] if "ignore_face_zonelets" in SubtractVolumesParams._default_params else (json_data["ignoreFaceZonelets"] if "ignoreFaceZonelets" in json_data else None)),
                        check_cutters if check_cutters is not None else ( SubtractVolumesParams._default_params["check_cutters"] if "check_cutters" in SubtractVolumesParams._default_params else (json_data["checkCutters"] if "checkCutters" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            ignore_face_zonelets: Iterable[int] = None,
            check_cutters: bool = None):
        """Set the default values of SubtractVolumesParams.

        Parameters
        ----------
        ignore_face_zonelets: Iterable[int], optional
            Face zonelet ids that subtract volumes should not remove.(For example, periodic or fluid cap zonelets)
        check_cutters: bool, optional
            Option to handle overlapping or intersecting cutter volumes.
        """
        args = locals()
        [SubtractVolumesParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SubtractVolumesParams.

        Examples
        --------
        >>> SubtractVolumesParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SubtractVolumesParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._ignore_face_zonelets is not None:
            json_data["ignoreFaceZonelets"] = self._ignore_face_zonelets
        if self._check_cutters is not None:
            json_data["checkCutters"] = self._check_cutters
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "ignore_face_zonelets :  %s\ncheck_cutters :  %s" % (self._ignore_face_zonelets, self._check_cutters)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def ignore_face_zonelets(self) -> Iterable[int]:
        """Face zonelet ids that subtract volumes should not remove.(For example, periodic or fluid cap zonelets)
        """
        return self._ignore_face_zonelets

    @ignore_face_zonelets.setter
    def ignore_face_zonelets(self, value: Iterable[int]):
        self._ignore_face_zonelets = value

    @property
    def check_cutters(self) -> bool:
        """Option to handle overlapping or intersecting cutter volumes.
        """
        return self._check_cutters

    @check_cutters.setter
    def check_cutters(self, value: bool):
        self._check_cutters = value

class SubtractVolumesResults(CoreObject):
    """Results of the volume subtract operation.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode):
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SubtractVolumesResults.

        Parameters
        ----------
        model: Model
            Model to create a SubtractVolumesResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the volume subtract operation.
        json_data: dict, optional
            JSON dictionary to create a SubtractVolumesResults object with provided parameters.

        Examples
        --------
        >>> subtract_volumes_results = prime.SubtractVolumesResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [error_code])
            if all_field_specified:
                self.__initialize(
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SubtractVolumesResults")
                    json_data = param_json["SubtractVolumesResults"] if "SubtractVolumesResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( SubtractVolumesResults._default_params["error_code"] if "error_code" in SubtractVolumesResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None):
        """Set the default values of SubtractVolumesResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the volume subtract operation.
        """
        args = locals()
        [SubtractVolumesResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SubtractVolumesResults.

        Examples
        --------
        >>> SubtractVolumesResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SubtractVolumesResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s" % (self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the volume subtract operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class StitchParams(CoreObject):
    """Parameters used for stitch operation.
    """
    _default_params = {}

    def __initialize(
            self,
            tolerance: float,
            use_absolute_tolerance: bool,
            remesh: bool,
            enable_multi_threading: bool,
            type: StitchType):
        self._tolerance = tolerance
        self._use_absolute_tolerance = use_absolute_tolerance
        self._remesh = remesh
        self._enable_multi_threading = enable_multi_threading
        self._type = StitchType(type)

    def __init__(
            self,
            model: CommunicationManager=None,
            tolerance: float = None,
            use_absolute_tolerance: bool = None,
            remesh: bool = None,
            enable_multi_threading: bool = None,
            type: StitchType = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the StitchParams.

        Parameters
        ----------
        model: Model
            Model to create a StitchParams object with default parameters.
        tolerance: float, optional
            Distance tolerance for stitching boundaries.
        use_absolute_tolerance: bool, optional
            True if tolerance provided is absolute value.
        remesh: bool, optional
            Remesh at stitch connection.
        enable_multi_threading: bool, optional
            Option to run stitch in parallel using multithread.
        type: StitchType, optional
            Stitch type depending on nature of surface boundary edges to be stitched.
        json_data: dict, optional
            JSON dictionary to create a StitchParams object with provided parameters.

        Examples
        --------
        >>> stitch_params = prime.StitchParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["tolerance"] if "tolerance" in json_data else None,
                json_data["useAbsoluteTolerance"] if "useAbsoluteTolerance" in json_data else None,
                json_data["remesh"] if "remesh" in json_data else None,
                json_data["enableMultiThreading"] if "enableMultiThreading" in json_data else None,
                StitchType(json_data["type"] if "type" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [tolerance, use_absolute_tolerance, remesh, enable_multi_threading, type])
            if all_field_specified:
                self.__initialize(
                    tolerance,
                    use_absolute_tolerance,
                    remesh,
                    enable_multi_threading,
                    type)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "StitchParams")
                    json_data = param_json["StitchParams"] if "StitchParams" in param_json else {}
                    self.__initialize(
                        tolerance if tolerance is not None else ( StitchParams._default_params["tolerance"] if "tolerance" in StitchParams._default_params else (json_data["tolerance"] if "tolerance" in json_data else None)),
                        use_absolute_tolerance if use_absolute_tolerance is not None else ( StitchParams._default_params["use_absolute_tolerance"] if "use_absolute_tolerance" in StitchParams._default_params else (json_data["useAbsoluteTolerance"] if "useAbsoluteTolerance" in json_data else None)),
                        remesh if remesh is not None else ( StitchParams._default_params["remesh"] if "remesh" in StitchParams._default_params else (json_data["remesh"] if "remesh" in json_data else None)),
                        enable_multi_threading if enable_multi_threading is not None else ( StitchParams._default_params["enable_multi_threading"] if "enable_multi_threading" in StitchParams._default_params else (json_data["enableMultiThreading"] if "enableMultiThreading" in json_data else None)),
                        type if type is not None else ( StitchParams._default_params["type"] if "type" in StitchParams._default_params else StitchType(json_data["type"] if "type" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            tolerance: float = None,
            use_absolute_tolerance: bool = None,
            remesh: bool = None,
            enable_multi_threading: bool = None,
            type: StitchType = None):
        """Set the default values of StitchParams.

        Parameters
        ----------
        tolerance: float, optional
            Distance tolerance for stitching boundaries.
        use_absolute_tolerance: bool, optional
            True if tolerance provided is absolute value.
        remesh: bool, optional
            Remesh at stitch connection.
        enable_multi_threading: bool, optional
            Option to run stitch in parallel using multithread.
        type: StitchType, optional
            Stitch type depending on nature of surface boundary edges to be stitched.
        """
        args = locals()
        [StitchParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of StitchParams.

        Examples
        --------
        >>> StitchParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in StitchParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._tolerance is not None:
            json_data["tolerance"] = self._tolerance
        if self._use_absolute_tolerance is not None:
            json_data["useAbsoluteTolerance"] = self._use_absolute_tolerance
        if self._remesh is not None:
            json_data["remesh"] = self._remesh
        if self._enable_multi_threading is not None:
            json_data["enableMultiThreading"] = self._enable_multi_threading
        if self._type is not None:
            json_data["type"] = self._type
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "tolerance :  %s\nuse_absolute_tolerance :  %s\nremesh :  %s\nenable_multi_threading :  %s\ntype :  %s" % (self._tolerance, self._use_absolute_tolerance, self._remesh, self._enable_multi_threading, self._type)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def tolerance(self) -> float:
        """Distance tolerance for stitching boundaries.
        """
        return self._tolerance

    @tolerance.setter
    def tolerance(self, value: float):
        self._tolerance = value

    @property
    def use_absolute_tolerance(self) -> bool:
        """True if tolerance provided is absolute value.
        """
        return self._use_absolute_tolerance

    @use_absolute_tolerance.setter
    def use_absolute_tolerance(self, value: bool):
        self._use_absolute_tolerance = value

    @property
    def remesh(self) -> bool:
        """Remesh at stitch connection.
        """
        return self._remesh

    @remesh.setter
    def remesh(self, value: bool):
        self._remesh = value

    @property
    def enable_multi_threading(self) -> bool:
        """Option to run stitch in parallel using multithread.
        """
        return self._enable_multi_threading

    @enable_multi_threading.setter
    def enable_multi_threading(self, value: bool):
        self._enable_multi_threading = value

    @property
    def type(self) -> StitchType:
        """Stitch type depending on nature of surface boundary edges to be stitched.
        """
        return self._type

    @type.setter
    def type(self, value: StitchType):
        self._type = value



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class AdvancedSurferSetup(enum.IntEnum):
    """Define advanced settings for remeshing operation.
    """
    NONE = 0
    """Option to define no advanced settings."""
    WRAPPER = 1
    """Option to define advanced settings for wrapper surfaces."""

class SurferParams(CoreObject):
    """Parameters used to generate surface mesh.
    """
    _default_params = {}

    def __initialize(
            self,
            max_angle: float,
            size_field_type: SizeFieldType,
            min_size: float,
            max_size: float,
            growth_rate: float,
            constant_size: float,
            generate_quads: bool,
            check_non_manifolds: bool,
            avoid_corner_triangles: bool,
            smooth_size_transition: bool,
            advanced_surfer_setup: AdvancedSurferSetup,
            project_on_geometry: bool,
            enable_multi_threading: bool):
        self._max_angle = max_angle
        self._size_field_type = SizeFieldType(size_field_type)
        self._min_size = min_size
        self._max_size = max_size
        self._growth_rate = growth_rate
        self._constant_size = constant_size
        self._generate_quads = generate_quads
        self._check_non_manifolds = check_non_manifolds
        self._avoid_corner_triangles = avoid_corner_triangles
        self._smooth_size_transition = smooth_size_transition
        self._advanced_surfer_setup = AdvancedSurferSetup(advanced_surfer_setup)
        self._project_on_geometry = project_on_geometry
        self._enable_multi_threading = enable_multi_threading

    def __init__(
            self,
            model: CommunicationManager=None,
            max_angle: float = None,
            size_field_type: SizeFieldType = None,
            min_size: float = None,
            max_size: float = None,
            growth_rate: float = None,
            constant_size: float = None,
            generate_quads: bool = None,
            check_non_manifolds: bool = None,
            avoid_corner_triangles: bool = None,
            smooth_size_transition: bool = None,
            advanced_surfer_setup: AdvancedSurferSetup = None,
            project_on_geometry: bool = None,
            enable_multi_threading: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurferParams.

        Parameters
        ----------
        model: Model
            Model to create a SurferParams object with default parameters.
        max_angle: float, optional
            Maximum feature angle limit to be used to identify and preserve features.
        size_field_type: SizeFieldType, optional
            Size field type used to generate surface mesh.
        min_size: float, optional
            Minimum size to be used in sizing for the surfer.
        max_size: float, optional
            Maximum size to be used in sizing for the surfer.
        growth_rate: float, optional
            Growth rate to be used to propagate sizes.
        constant_size: float, optional
            Size used in constant size surface meshing.
        generate_quads: bool, optional
            Option to generate quadrilateral surface mesh.
        check_non_manifolds: bool, optional
            Option to avoid new non-manifolds(multi-connection) if generated in surface mesh.
        avoid_corner_triangles: bool, optional
            Option to avoid corner triangles(with all three boundary nodes) generated.
        smooth_size_transition: bool, optional
            Option to generate mesh with smooth size transition from neighbors of selected surfaces. This includes neighboring face edge sizes in sizing provided for surface meshing to achieve smooth size transition.
        advanced_surfer_setup: AdvancedSurferSetup, optional
            Option to define advanced settings for remeshing operation.
        project_on_geometry: bool, optional
            Option to project on CAD geometry when meshing.
        enable_multi_threading: bool, optional
            Option to perform surface meshing in parallel using multithreads.
        json_data: dict, optional
            JSON dictionary to create a SurferParams object with provided parameters.

        Examples
        --------
        >>> surfer_params = prime.SurferParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["maxAngle"] if "maxAngle" in json_data else None,
                SizeFieldType(json_data["sizeFieldType"] if "sizeFieldType" in json_data else None),
                json_data["minSize"] if "minSize" in json_data else None,
                json_data["maxSize"] if "maxSize" in json_data else None,
                json_data["growthRate"] if "growthRate" in json_data else None,
                json_data["constantSize"] if "constantSize" in json_data else None,
                json_data["generateQuads"] if "generateQuads" in json_data else None,
                json_data["checkNonManifolds"] if "checkNonManifolds" in json_data else None,
                json_data["avoidCornerTriangles"] if "avoidCornerTriangles" in json_data else None,
                json_data["smoothSizeTransition"] if "smoothSizeTransition" in json_data else None,
                AdvancedSurferSetup(json_data["advancedSurferSetup"] if "advancedSurferSetup" in json_data else None),
                json_data["projectOnGeometry"] if "projectOnGeometry" in json_data else None,
                json_data["enableMultiThreading"] if "enableMultiThreading" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [max_angle, size_field_type, min_size, max_size, growth_rate, constant_size, generate_quads, check_non_manifolds, avoid_corner_triangles, smooth_size_transition, advanced_surfer_setup, project_on_geometry, enable_multi_threading])
            if all_field_specified:
                self.__initialize(
                    max_angle,
                    size_field_type,
                    min_size,
                    max_size,
                    growth_rate,
                    constant_size,
                    generate_quads,
                    check_non_manifolds,
                    avoid_corner_triangles,
                    smooth_size_transition,
                    advanced_surfer_setup,
                    project_on_geometry,
                    enable_multi_threading)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurferParams")
                    json_data = param_json["SurferParams"] if "SurferParams" in param_json else {}
                    self.__initialize(
                        max_angle if max_angle is not None else ( SurferParams._default_params["max_angle"] if "max_angle" in SurferParams._default_params else (json_data["maxAngle"] if "maxAngle" in json_data else None)),
                        size_field_type if size_field_type is not None else ( SurferParams._default_params["size_field_type"] if "size_field_type" in SurferParams._default_params else SizeFieldType(json_data["sizeFieldType"] if "sizeFieldType" in json_data else None)),
                        min_size if min_size is not None else ( SurferParams._default_params["min_size"] if "min_size" in SurferParams._default_params else (json_data["minSize"] if "minSize" in json_data else None)),
                        max_size if max_size is not None else ( SurferParams._default_params["max_size"] if "max_size" in SurferParams._default_params else (json_data["maxSize"] if "maxSize" in json_data else None)),
                        growth_rate if growth_rate is not None else ( SurferParams._default_params["growth_rate"] if "growth_rate" in SurferParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)),
                        constant_size if constant_size is not None else ( SurferParams._default_params["constant_size"] if "constant_size" in SurferParams._default_params else (json_data["constantSize"] if "constantSize" in json_data else None)),
                        generate_quads if generate_quads is not None else ( SurferParams._default_params["generate_quads"] if "generate_quads" in SurferParams._default_params else (json_data["generateQuads"] if "generateQuads" in json_data else None)),
                        check_non_manifolds if check_non_manifolds is not None else ( SurferParams._default_params["check_non_manifolds"] if "check_non_manifolds" in SurferParams._default_params else (json_data["checkNonManifolds"] if "checkNonManifolds" in json_data else None)),
                        avoid_corner_triangles if avoid_corner_triangles is not None else ( SurferParams._default_params["avoid_corner_triangles"] if "avoid_corner_triangles" in SurferParams._default_params else (json_data["avoidCornerTriangles"] if "avoidCornerTriangles" in json_data else None)),
                        smooth_size_transition if smooth_size_transition is not None else ( SurferParams._default_params["smooth_size_transition"] if "smooth_size_transition" in SurferParams._default_params else (json_data["smoothSizeTransition"] if "smoothSizeTransition" in json_data else None)),
                        advanced_surfer_setup if advanced_surfer_setup is not None else ( SurferParams._default_params["advanced_surfer_setup"] if "advanced_surfer_setup" in SurferParams._default_params else AdvancedSurferSetup(json_data["advancedSurferSetup"] if "advancedSurferSetup" in json_data else None)),
                        project_on_geometry if project_on_geometry is not None else ( SurferParams._default_params["project_on_geometry"] if "project_on_geometry" in SurferParams._default_params else (json_data["projectOnGeometry"] if "projectOnGeometry" in json_data else None)),
                        enable_multi_threading if enable_multi_threading is not None else ( SurferParams._default_params["enable_multi_threading"] if "enable_multi_threading" in SurferParams._default_params else (json_data["enableMultiThreading"] if "enableMultiThreading" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            max_angle: float = None,
            size_field_type: SizeFieldType = None,
            min_size: float = None,
            max_size: float = None,
            growth_rate: float = None,
            constant_size: float = None,
            generate_quads: bool = None,
            check_non_manifolds: bool = None,
            avoid_corner_triangles: bool = None,
            smooth_size_transition: bool = None,
            advanced_surfer_setup: AdvancedSurferSetup = None,
            project_on_geometry: bool = None,
            enable_multi_threading: bool = None):
        """Set the default values of SurferParams.

        Parameters
        ----------
        max_angle: float, optional
            Maximum feature angle limit to be used to identify and preserve features.
        size_field_type: SizeFieldType, optional
            Size field type used to generate surface mesh.
        min_size: float, optional
            Minimum size to be used in sizing for the surfer.
        max_size: float, optional
            Maximum size to be used in sizing for the surfer.
        growth_rate: float, optional
            Growth rate to be used to propagate sizes.
        constant_size: float, optional
            Size used in constant size surface meshing.
        generate_quads: bool, optional
            Option to generate quadrilateral surface mesh.
        check_non_manifolds: bool, optional
            Option to avoid new non-manifolds(multi-connection) if generated in surface mesh.
        avoid_corner_triangles: bool, optional
            Option to avoid corner triangles(with all three boundary nodes) generated.
        smooth_size_transition: bool, optional
            Option to generate mesh with smooth size transition from neighbors of selected surfaces. This includes neighboring face edge sizes in sizing provided for surface meshing to achieve smooth size transition.
        advanced_surfer_setup: AdvancedSurferSetup, optional
            Option to define advanced settings for remeshing operation.
        project_on_geometry: bool, optional
            Option to project on CAD geometry when meshing.
        enable_multi_threading: bool, optional
            Option to perform surface meshing in parallel using multithreads.
        """
        args = locals()
        [SurferParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurferParams.

        Examples
        --------
        >>> SurferParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurferParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._max_angle is not None:
            json_data["maxAngle"] = self._max_angle
        if self._size_field_type is not None:
            json_data["sizeFieldType"] = self._size_field_type
        if self._min_size is not None:
            json_data["minSize"] = self._min_size
        if self._max_size is not None:
            json_data["maxSize"] = self._max_size
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        if self._constant_size is not None:
            json_data["constantSize"] = self._constant_size
        if self._generate_quads is not None:
            json_data["generateQuads"] = self._generate_quads
        if self._check_non_manifolds is not None:
            json_data["checkNonManifolds"] = self._check_non_manifolds
        if self._avoid_corner_triangles is not None:
            json_data["avoidCornerTriangles"] = self._avoid_corner_triangles
        if self._smooth_size_transition is not None:
            json_data["smoothSizeTransition"] = self._smooth_size_transition
        if self._advanced_surfer_setup is not None:
            json_data["advancedSurferSetup"] = self._advanced_surfer_setup
        if self._project_on_geometry is not None:
            json_data["projectOnGeometry"] = self._project_on_geometry
        if self._enable_multi_threading is not None:
            json_data["enableMultiThreading"] = self._enable_multi_threading
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "max_angle :  %s\nsize_field_type :  %s\nmin_size :  %s\nmax_size :  %s\ngrowth_rate :  %s\nconstant_size :  %s\ngenerate_quads :  %s\ncheck_non_manifolds :  %s\navoid_corner_triangles :  %s\nsmooth_size_transition :  %s\nadvanced_surfer_setup :  %s\nproject_on_geometry :  %s\nenable_multi_threading :  %s" % (self._max_angle, self._size_field_type, self._min_size, self._max_size, self._growth_rate, self._constant_size, self._generate_quads, self._check_non_manifolds, self._avoid_corner_triangles, self._smooth_size_transition, self._advanced_surfer_setup, self._project_on_geometry, self._enable_multi_threading)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def max_angle(self) -> float:
        """Maximum feature angle limit to be used to identify and preserve features.
        """
        return self._max_angle

    @max_angle.setter
    def max_angle(self, value: float):
        self._max_angle = value

    @property
    def size_field_type(self) -> SizeFieldType:
        """Size field type used to generate surface mesh.
        """
        return self._size_field_type

    @size_field_type.setter
    def size_field_type(self, value: SizeFieldType):
        self._size_field_type = value

    @property
    def min_size(self) -> float:
        """Minimum size to be used in sizing for the surfer.
        """
        return self._min_size

    @min_size.setter
    def min_size(self, value: float):
        self._min_size = value

    @property
    def max_size(self) -> float:
        """Maximum size to be used in sizing for the surfer.
        """
        return self._max_size

    @max_size.setter
    def max_size(self, value: float):
        self._max_size = value

    @property
    def growth_rate(self) -> float:
        """Growth rate to be used to propagate sizes.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

    @property
    def constant_size(self) -> float:
        """Size used in constant size surface meshing.
        """
        return self._constant_size

    @constant_size.setter
    def constant_size(self, value: float):
        self._constant_size = value

    @property
    def generate_quads(self) -> bool:
        """Option to generate quadrilateral surface mesh.
        """
        return self._generate_quads

    @generate_quads.setter
    def generate_quads(self, value: bool):
        self._generate_quads = value

    @property
    def check_non_manifolds(self) -> bool:
        """Option to avoid new non-manifolds(multi-connection) if generated in surface mesh.
        """
        return self._check_non_manifolds

    @check_non_manifolds.setter
    def check_non_manifolds(self, value: bool):
        self._check_non_manifolds = value

    @property
    def avoid_corner_triangles(self) -> bool:
        """Option to avoid corner triangles(with all three boundary nodes) generated.
        """
        return self._avoid_corner_triangles

    @avoid_corner_triangles.setter
    def avoid_corner_triangles(self, value: bool):
        self._avoid_corner_triangles = value

    @property
    def smooth_size_transition(self) -> bool:
        """Option to generate mesh with smooth size transition from neighbors of selected surfaces. This includes neighboring face edge sizes in sizing provided for surface meshing to achieve smooth size transition.

        Notes
         -----
          - Input facets or mesh with finer sizes compared to neighboring face edge sizes are required for this option to work.
          - Valid min, max sizes and growth rate are required to include the neighboring face edges sizes in sizing.
        """
        return self._smooth_size_transition

    @smooth_size_transition.setter
    def smooth_size_transition(self, value: bool):
        self._smooth_size_transition = value

    @property
    def advanced_surfer_setup(self) -> AdvancedSurferSetup:
        """Option to define advanced settings for remeshing operation.
        """
        return self._advanced_surfer_setup

    @advanced_surfer_setup.setter
    def advanced_surfer_setup(self, value: AdvancedSurferSetup):
        self._advanced_surfer_setup = value

    @property
    def project_on_geometry(self) -> bool:
        """Option to project on CAD geometry when meshing.
        """
        return self._project_on_geometry

    @project_on_geometry.setter
    def project_on_geometry(self, value: bool):
        self._project_on_geometry = value

    @property
    def enable_multi_threading(self) -> bool:
        """Option to perform surface meshing in parallel using multithreads.
        """
        return self._enable_multi_threading

    @enable_multi_threading.setter
    def enable_multi_threading(self, value: bool):
        self._enable_multi_threading = value

class SurferResults(CoreObject):
    """Results associated with the surface mesh.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            topofaces_not_projected_on_geometry: Iterable[int]):
        self._error_code = ErrorCode(error_code)
        self._topofaces_not_projected_on_geometry = topofaces_not_projected_on_geometry if isinstance(topofaces_not_projected_on_geometry, np.ndarray) else np.array(topofaces_not_projected_on_geometry, dtype=np.int32) if topofaces_not_projected_on_geometry is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            topofaces_not_projected_on_geometry: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurferResults.

        Parameters
        ----------
        model: Model
            Model to create a SurferResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        topofaces_not_projected_on_geometry: Iterable[int], optional
            Ids of topofaces projected to facets instead of CAD geometry, when projectOnGeometry is enabled.
        json_data: dict, optional
            JSON dictionary to create a SurferResults object with provided parameters.

        Examples
        --------
        >>> surfer_results = prime.SurferResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["topofacesNotProjectedOnGeometry"] if "topofacesNotProjectedOnGeometry" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, topofaces_not_projected_on_geometry])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    topofaces_not_projected_on_geometry)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurferResults")
                    json_data = param_json["SurferResults"] if "SurferResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( SurferResults._default_params["error_code"] if "error_code" in SurferResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        topofaces_not_projected_on_geometry if topofaces_not_projected_on_geometry is not None else ( SurferResults._default_params["topofaces_not_projected_on_geometry"] if "topofaces_not_projected_on_geometry" in SurferResults._default_params else (json_data["topofacesNotProjectedOnGeometry"] if "topofacesNotProjectedOnGeometry" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            topofaces_not_projected_on_geometry: Iterable[int] = None):
        """Set the default values of SurferResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        topofaces_not_projected_on_geometry: Iterable[int], optional
            Ids of topofaces projected to facets instead of CAD geometry, when projectOnGeometry is enabled.
        """
        args = locals()
        [SurferResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurferResults.

        Examples
        --------
        >>> SurferResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurferResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._topofaces_not_projected_on_geometry is not None:
            json_data["topofacesNotProjectedOnGeometry"] = self._topofaces_not_projected_on_geometry
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\ntopofaces_not_projected_on_geometry :  %s" % (self._error_code, self._topofaces_not_projected_on_geometry)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def topofaces_not_projected_on_geometry(self) -> Iterable[int]:
        """Ids of topofaces projected to facets instead of CAD geometry, when projectOnGeometry is enabled.
        """
        return self._topofaces_not_projected_on_geometry

    @topofaces_not_projected_on_geometry.setter
    def topofaces_not_projected_on_geometry(self, value: Iterable[int]):
        self._topofaces_not_projected_on_geometry = value

class LocalSurferParams(CoreObject):
    """Parameters to perform local surface remeshing.
    """
    _default_params = {}

    def __initialize(
            self,
            min_angle: float,
            max_angle: float,
            size_field_type: SizeFieldType,
            min_size: float,
            max_size: float,
            growth_rate: float,
            constant_size: float,
            smooth_boundary: bool,
            n_rings: int):
        self._min_angle = min_angle
        self._max_angle = max_angle
        self._size_field_type = SizeFieldType(size_field_type)
        self._min_size = min_size
        self._max_size = max_size
        self._growth_rate = growth_rate
        self._constant_size = constant_size
        self._smooth_boundary = smooth_boundary
        self._n_rings = n_rings

    def __init__(
            self,
            model: CommunicationManager=None,
            min_angle: float = None,
            max_angle: float = None,
            size_field_type: SizeFieldType = None,
            min_size: float = None,
            max_size: float = None,
            growth_rate: float = None,
            constant_size: float = None,
            smooth_boundary: bool = None,
            n_rings: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the LocalSurferParams.

        Parameters
        ----------
        model: Model
            Model to create a LocalSurferParams object with default parameters.
        min_angle: float, optional
            Minimum feature angle limit used to identify and preserve features.
        max_angle: float, optional
            Maximum feature angle limit used to identify and preserve features.
        size_field_type: SizeFieldType, optional
            Size field type used to generate surface mesh.
        min_size: float, optional
            Minimum size to be used in sizing for the surfer.
        max_size: float, optional
            Maximum size to be used in sizing for the surfer.
        growth_rate: float, optional
            Growth rate to be used to propagate sizes.
        constant_size: float, optional
            Constant size to be used in case of constant size field.
        smooth_boundary: bool, optional
            Option to extend local selection to get smooth boundary of selected elements.
        n_rings: int, optional
            Number of rings to extend the registered face selection for remeshing.
        json_data: dict, optional
            JSON dictionary to create a LocalSurferParams object with provided parameters.

        Examples
        --------
        >>> local_surfer_params = prime.LocalSurferParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["minAngle"] if "minAngle" in json_data else None,
                json_data["maxAngle"] if "maxAngle" in json_data else None,
                SizeFieldType(json_data["sizeFieldType"] if "sizeFieldType" in json_data else None),
                json_data["minSize"] if "minSize" in json_data else None,
                json_data["maxSize"] if "maxSize" in json_data else None,
                json_data["growthRate"] if "growthRate" in json_data else None,
                json_data["constantSize"] if "constantSize" in json_data else None,
                json_data["smoothBoundary"] if "smoothBoundary" in json_data else None,
                json_data["nRings"] if "nRings" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [min_angle, max_angle, size_field_type, min_size, max_size, growth_rate, constant_size, smooth_boundary, n_rings])
            if all_field_specified:
                self.__initialize(
                    min_angle,
                    max_angle,
                    size_field_type,
                    min_size,
                    max_size,
                    growth_rate,
                    constant_size,
                    smooth_boundary,
                    n_rings)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "LocalSurferParams")
                    json_data = param_json["LocalSurferParams"] if "LocalSurferParams" in param_json else {}
                    self.__initialize(
                        min_angle if min_angle is not None else ( LocalSurferParams._default_params["min_angle"] if "min_angle" in LocalSurferParams._default_params else (json_data["minAngle"] if "minAngle" in json_data else None)),
                        max_angle if max_angle is not None else ( LocalSurferParams._default_params["max_angle"] if "max_angle" in LocalSurferParams._default_params else (json_data["maxAngle"] if "maxAngle" in json_data else None)),
                        size_field_type if size_field_type is not None else ( LocalSurferParams._default_params["size_field_type"] if "size_field_type" in LocalSurferParams._default_params else SizeFieldType(json_data["sizeFieldType"] if "sizeFieldType" in json_data else None)),
                        min_size if min_size is not None else ( LocalSurferParams._default_params["min_size"] if "min_size" in LocalSurferParams._default_params else (json_data["minSize"] if "minSize" in json_data else None)),
                        max_size if max_size is not None else ( LocalSurferParams._default_params["max_size"] if "max_size" in LocalSurferParams._default_params else (json_data["maxSize"] if "maxSize" in json_data else None)),
                        growth_rate if growth_rate is not None else ( LocalSurferParams._default_params["growth_rate"] if "growth_rate" in LocalSurferParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)),
                        constant_size if constant_size is not None else ( LocalSurferParams._default_params["constant_size"] if "constant_size" in LocalSurferParams._default_params else (json_data["constantSize"] if "constantSize" in json_data else None)),
                        smooth_boundary if smooth_boundary is not None else ( LocalSurferParams._default_params["smooth_boundary"] if "smooth_boundary" in LocalSurferParams._default_params else (json_data["smoothBoundary"] if "smoothBoundary" in json_data else None)),
                        n_rings if n_rings is not None else ( LocalSurferParams._default_params["n_rings"] if "n_rings" in LocalSurferParams._default_params else (json_data["nRings"] if "nRings" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            min_angle: float = None,
            max_angle: float = None,
            size_field_type: SizeFieldType = None,
            min_size: float = None,
            max_size: float = None,
            growth_rate: float = None,
            constant_size: float = None,
            smooth_boundary: bool = None,
            n_rings: int = None):
        """Set the default values of LocalSurferParams.

        Parameters
        ----------
        min_angle: float, optional
            Minimum feature angle limit used to identify and preserve features.
        max_angle: float, optional
            Maximum feature angle limit used to identify and preserve features.
        size_field_type: SizeFieldType, optional
            Size field type used to generate surface mesh.
        min_size: float, optional
            Minimum size to be used in sizing for the surfer.
        max_size: float, optional
            Maximum size to be used in sizing for the surfer.
        growth_rate: float, optional
            Growth rate to be used to propagate sizes.
        constant_size: float, optional
            Constant size to be used in case of constant size field.
        smooth_boundary: bool, optional
            Option to extend local selection to get smooth boundary of selected elements.
        n_rings: int, optional
            Number of rings to extend the registered face selection for remeshing.
        """
        args = locals()
        [LocalSurferParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of LocalSurferParams.

        Examples
        --------
        >>> LocalSurferParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in LocalSurferParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._min_angle is not None:
            json_data["minAngle"] = self._min_angle
        if self._max_angle is not None:
            json_data["maxAngle"] = self._max_angle
        if self._size_field_type is not None:
            json_data["sizeFieldType"] = self._size_field_type
        if self._min_size is not None:
            json_data["minSize"] = self._min_size
        if self._max_size is not None:
            json_data["maxSize"] = self._max_size
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        if self._constant_size is not None:
            json_data["constantSize"] = self._constant_size
        if self._smooth_boundary is not None:
            json_data["smoothBoundary"] = self._smooth_boundary
        if self._n_rings is not None:
            json_data["nRings"] = self._n_rings
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "min_angle :  %s\nmax_angle :  %s\nsize_field_type :  %s\nmin_size :  %s\nmax_size :  %s\ngrowth_rate :  %s\nconstant_size :  %s\nsmooth_boundary :  %s\nn_rings :  %s" % (self._min_angle, self._max_angle, self._size_field_type, self._min_size, self._max_size, self._growth_rate, self._constant_size, self._smooth_boundary, self._n_rings)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def min_angle(self) -> float:
        """Minimum feature angle limit used to identify and preserve features.
        """
        return self._min_angle

    @min_angle.setter
    def min_angle(self, value: float):
        self._min_angle = value

    @property
    def max_angle(self) -> float:
        """Maximum feature angle limit used to identify and preserve features.
        """
        return self._max_angle

    @max_angle.setter
    def max_angle(self, value: float):
        self._max_angle = value

    @property
    def size_field_type(self) -> SizeFieldType:
        """Size field type used to generate surface mesh.
        """
        return self._size_field_type

    @size_field_type.setter
    def size_field_type(self, value: SizeFieldType):
        self._size_field_type = value

    @property
    def min_size(self) -> float:
        """Minimum size to be used in sizing for the surfer.
        """
        return self._min_size

    @min_size.setter
    def min_size(self, value: float):
        self._min_size = value

    @property
    def max_size(self) -> float:
        """Maximum size to be used in sizing for the surfer.
        """
        return self._max_size

    @max_size.setter
    def max_size(self, value: float):
        self._max_size = value

    @property
    def growth_rate(self) -> float:
        """Growth rate to be used to propagate sizes.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

    @property
    def constant_size(self) -> float:
        """Constant size to be used in case of constant size field.
        """
        return self._constant_size

    @constant_size.setter
    def constant_size(self, value: float):
        self._constant_size = value

    @property
    def smooth_boundary(self) -> bool:
        """Option to extend local selection to get smooth boundary of selected elements.
        """
        return self._smooth_boundary

    @smooth_boundary.setter
    def smooth_boundary(self, value: bool):
        self._smooth_boundary = value

    @property
    def n_rings(self) -> int:
        """Number of rings to extend the registered face selection for remeshing.
        """
        return self._n_rings

    @n_rings.setter
    def n_rings(self, value: int):
        self._n_rings = value

class LocalSurferResults(CoreObject):
    """Results associated with the local surface mesh.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            warning_codes: List[WarningCode]):
        self._error_code = ErrorCode(error_code)
        self._warning_codes = warning_codes

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the LocalSurferResults.

        Parameters
        ----------
        model: Model
            Model to create a LocalSurferResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the operation.
        json_data: dict, optional
            JSON dictionary to create a LocalSurferResults object with provided parameters.

        Examples
        --------
        >>> local_surfer_results = prime.LocalSurferResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                [WarningCode(data) for data in json_data["warningCodes"]] if "warningCodes" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, warning_codes])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    warning_codes)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "LocalSurferResults")
                    json_data = param_json["LocalSurferResults"] if "LocalSurferResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( LocalSurferResults._default_params["error_code"] if "error_code" in LocalSurferResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        warning_codes if warning_codes is not None else ( LocalSurferResults._default_params["warning_codes"] if "warning_codes" in LocalSurferResults._default_params else [WarningCode(data) for data in (json_data["warningCodes"] if "warningCodes" in json_data else None)]))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None):
        """Set the default values of LocalSurferResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the operation.
        """
        args = locals()
        [LocalSurferResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of LocalSurferResults.

        Examples
        --------
        >>> LocalSurferResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in LocalSurferResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._warning_codes is not None:
            json_data["warningCodes"] = [data for data in self._warning_codes]
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nwarning_codes :  %s" % (self._error_code, '[' + ''.join('\n' + str(data) for data in self._warning_codes) + ']')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def warning_codes(self) -> List[WarningCode]:
        """Warning codes associated with the operation.
        """
        return self._warning_codes

    @warning_codes.setter
    def warning_codes(self, value: List[WarningCode]):
        self._warning_codes = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class PeriodicControl(CoreObject):
    """Periodic controls provide settings for the recovery of periodic surfaces.

    A periodic control is specified by the scope (source surfaces) and the transformation parameters: the center, axis and angle.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize PeriodicControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def get_params(self) -> PeriodicControlParams:
        """ Get the parameters of the periodic control.


        Returns
        -------
        PeriodicControlParams
            Return parameters of the periodic control.


        Examples
        --------
        >>> params = periodic_control.get_params()

        """
        args = {}
        command_name = "PrimeMesh::PeriodicControl/GetParams"
        self._model._print_logs_before_command("get_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_params", PeriodicControlParams(model = self._model, json_data = result))
        return PeriodicControlParams(model = self._model, json_data = result)

    def set_params(self, periodic_control_params : PeriodicControlParams) -> SetParamsResults:
        """ Set the periodic control parameters.


        Parameters
        ----------
        periodic_control_params : PeriodicControlParams
            Parameters to control periodic surface recovery.

        Examples
        --------
        >>> periodic_control.set_params(
        >>>                  PeriodicControlParams(model=model,
        >>>                     center=[0,0,0], axis=[0,1,0], angle=180))

        """
        if not isinstance(periodic_control_params, PeriodicControlParams):
            raise TypeError("Invalid argument type passed for periodic_control_params, valid argument type is PeriodicControlParams.")
        args = {"periodic_control_params" : periodic_control_params._jsonify()}
        command_name = "PrimeMesh::PeriodicControl/SetParams"
        self._model._print_logs_before_command("set_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_params", SetParamsResults(model = self._model, json_data = result))
        return SetParamsResults(model = self._model, json_data = result)

    def set_suggested_name(self, name : str) -> SetNameResults:
        """ Sets the unique name for the periodic control based on the suggested name.


        Parameters
        ----------
        name : str
            Suggested name for the periodic control.

        Returns
        -------
        SetNameResults
            Returns the name of the periodic control.


        Examples
        --------
        >>> periodic_control.set_suggested_name("control1")

        """
        if not isinstance(name, str):
            raise TypeError("Invalid argument type passed for name, valid argument type is str.")
        args = {"name" : name}
        command_name = "PrimeMesh::PeriodicControl/SetSuggestedName"
        self._model._print_logs_before_command("set_suggested_name", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_suggested_name", SetNameResults(model = self._model, json_data = result))
        return SetNameResults(model = self._model, json_data = result)

    def get_scope(self) -> ScopeDefinition:
        """ Gets the scope for the periodic control.


        Returns
        -------
        ScopeDefinition
            Returns the scope of the periodic control.


        Examples
        --------
        >>> scope_definition = periodic_control.get_scope()

        """
        args = {}
        command_name = "PrimeMesh::PeriodicControl/GetScope"
        self._model._print_logs_before_command("get_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    def set_scope(self, scope : ScopeDefinition) -> SetScopeResults:
        """ Sets the scope for periodic control to evaluate.

        Periodic Control uses scope to evaluate entities for which periodic surface recovery must be carried out.

        Parameters
        ----------
        scope : ScopeDefinition
            ScopeDefinition to scope entities for periodic surface recovery.

        Returns
        -------
        SetScopeResults
            Returns the SetScopeResults.


        Examples
        --------
        >>> surface_scope = prime.ScopeDefinition(model=model,
        >>>                     entity_type=prime.ScopeEntity.FACEZONELETS,
        >>>                     evaluation_type=prime.ScopeEvaluationType.LABELS,
        >>>                     label_expression="periodic-1")
        >>> periodic_control.set_scope(surface_scope)

        """
        if not isinstance(scope, ScopeDefinition):
            raise TypeError("Invalid argument type passed for scope, valid argument type is ScopeDefinition.")
        args = {"scope" : scope._jsonify()}
        command_name = "PrimeMesh::PeriodicControl/SetScope"
        self._model._print_logs_before_command("set_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def get_summary(self, params : PeriodicControlSummaryParams) -> PeriodicControlSummaryResult:
        """ Get the periodic control summary along with the evaluated scope for the provided parameters.


        Parameters
        ----------
        params : PeriodicControlSummaryParams
            Periodic control summary parameters.

        Returns
        -------
        PeriodicControlSummaryResult
            Return the PeriodicControlSummaryResult.

        Examples
        --------
        >>> results = periodic_control.get_summary(prime.PeriodicControlSummaryParams(model=model))

        """
        if not isinstance(params, PeriodicControlSummaryParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is PeriodicControlSummaryParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::PeriodicControl/GetSummary"
        self._model._print_logs_before_command("get_summary", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_summary", PeriodicControlSummaryResult(model = self._model, json_data = result))
        return PeriodicControlSummaryResult(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of PeriodicControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of PeriodicControl."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class SplitParams(CoreObject):
    """Parameters to split face elements at longest edge.
    """
    _default_params = {}

    def __initialize(
            self,
            split_ratio: float):
        self._split_ratio = split_ratio

    def __init__(
            self,
            model: CommunicationManager=None,
            split_ratio: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SplitParams.

        Parameters
        ----------
        model: Model
            Model to create a SplitParams object with default parameters.
        split_ratio: float, optional
            Minimum ratio of split edge length to original edge length.
        json_data: dict, optional
            JSON dictionary to create a SplitParams object with provided parameters.

        Examples
        --------
        >>> split_params = prime.SplitParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["splitRatio"] if "splitRatio" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [split_ratio])
            if all_field_specified:
                self.__initialize(
                    split_ratio)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SplitParams")
                    json_data = param_json["SplitParams"] if "SplitParams" in param_json else {}
                    self.__initialize(
                        split_ratio if split_ratio is not None else ( SplitParams._default_params["split_ratio"] if "split_ratio" in SplitParams._default_params else (json_data["splitRatio"] if "splitRatio" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            split_ratio: float = None):
        """Set the default values of SplitParams.

        Parameters
        ----------
        split_ratio: float, optional
            Minimum ratio of split edge length to original edge length.
        """
        args = locals()
        [SplitParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SplitParams.

        Examples
        --------
        >>> SplitParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SplitParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._split_ratio is not None:
            json_data["splitRatio"] = self._split_ratio
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "split_ratio :  %s" % (self._split_ratio)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def split_ratio(self) -> float:
        """Minimum ratio of split edge length to original edge length.
        """
        return self._split_ratio

    @split_ratio.setter
    def split_ratio(self, value: float):
        self._split_ratio = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class Scaffolder(CoreObject):
    """Scaffolder is used for achieving connections in structures made of sheets and beams. Solid bodies should be suppressed before applying scaffolding.

    """

    def __init__(self, model: CommunicationManager, part_id: int):
        """ Initialize Scaffolder  """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::Scaffolder/Construct"
        args = {"ModelID" : model._object_id , "PartID" : part_id, "MaxID" : -1}
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for Scaffolder. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for Scaffolder. """
        command_name = "PrimeMesh::Scaffolder/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def scaffold_topo_faces_and_beams(self, topo_faces : Iterable[int], topo_beams : Iterable[int], params : ScaffolderParams) -> ScaffolderResults:
        """ Scaffold faces and beams with provided parameters.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of topology faces.
        topo_beams : Iterable[int]
            Ids of topology edges for beams.
        params : ScaffolderParams
            Scaffolding parameters.

        Returns
        -------
        ScaffolderResults
            Return results in ScaffolderResults structure.


        Examples
        --------
        >>> results = scaffolder.ScaffoldTopoFacesAndBeams([1,2], [7,8], params)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        if not isinstance(topo_beams, Iterable):
            raise TypeError("Invalid argument type passed for topo_beams, valid argument type is Iterable[int].")
        if not isinstance(params, ScaffolderParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ScaffolderParams.")
        args = {"topo_faces" : topo_faces,
        "topo_beams" : topo_beams,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Scaffolder/ScaffoldTopoFacesAndBeams"
        self._model._print_logs_before_command("scaffold_topo_faces_and_beams", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("scaffold_topo_faces_and_beams", ScaffolderResults(model = self._model, json_data = result))
        return ScaffolderResults(model = self._model, json_data = result)

    def split_topo_faces_by_mesh_region(self, topo_faces : Iterable[int]) -> ScaffolderSplitResults:
        """ Split input topofaces by mesh region.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of topology faces.

        Returns
        -------
        ScaffolderSplitResults
            Return results in ScaffolderSplitResults type.


        Examples
        --------
        >>> results = scaffolder.split_topo_faces_by_mesh_region([1,2,7,8])

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::Scaffolder/SplitTopoFacesByMeshRegion"
        self._model._print_logs_before_command("split_topo_faces_by_mesh_region", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("split_topo_faces_by_mesh_region", ScaffolderSplitResults(model = self._model, json_data = result))
        return ScaffolderSplitResults(model = self._model, json_data = result)

    def merge_overlapping_topo_faces(self, topo_faces : Iterable[int], params : ScaffolderParams) -> ScaffolderMergeResults:
        """ Merge overlapping topofaces.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of topology faces.
        params : ScaffolderParams
            Scaffolder parameters.

        Returns
        -------
        ScaffolderMergeResults
            Return results in ScaffolderMergeResults.


        Examples
        --------
        >>> results = scaffolder.MergeOverlappingTopoFaces([1,2,7,8], params)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        if not isinstance(params, ScaffolderParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ScaffolderParams.")
        args = {"topo_faces" : topo_faces,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Scaffolder/MergeOverlappingTopoFaces"
        self._model._print_logs_before_command("merge_overlapping_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("merge_overlapping_topo_faces", ScaffolderMergeResults(model = self._model, json_data = result))
        return ScaffolderMergeResults(model = self._model, json_data = result)

    def delete_shadowed_topo_faces(self, topo_faces : Iterable[int], params : VolumetricScaffolderParams) -> ScaffolderMergeResults:
        """ Delete fully shadowed topofaces.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of input topofaces.
        params : VolumetricScaffolderParams
            Volumetric scaffolder parameters.

        Returns
        -------
        ScaffolderMergeResults
            Return results in ScaffolderMergeResults.


        Examples
        --------
        >>> results = scaffolder.delete_shadowed_topo_faces([1,2,3,4,5], params)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        if not isinstance(params, VolumetricScaffolderParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is VolumetricScaffolderParams.")
        args = {"topo_faces" : topo_faces,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Scaffolder/DeleteShadowedTopoFaces"
        self._model._print_logs_before_command("delete_shadowed_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_shadowed_topo_faces", ScaffolderMergeResults(model = self._model, json_data = result))
        return ScaffolderMergeResults(model = self._model, json_data = result)



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class SizeField(CoreObject):
    """The size field is computed based on the size control defined.

    You can remesh surfaces and edges based on the size field.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize SizeField """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::SizeField/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for SizeField. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for SizeField. """
        command_name = "PrimeMesh::SizeField/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def compute_volumetric(self, size_control_ids : Iterable[int], volumetric_sizefield_params : VolumetricSizeFieldComputeParams) -> VolumetricSizeFieldComputeResults:
        """ Computes the volumetric size field using given size control ids.


        Parameters
        ----------
        size_control_ids : Iterable[int]
            Ids of size controls.

        Examples
        --------
        >>> size_field.compute_volumetric(
        >>>           [size_control.id for size_control in model.control_data.size_controls], volumetric_sizefield_params))

        """
        if not isinstance(size_control_ids, Iterable):
            raise TypeError("Invalid argument type passed for size_control_ids, valid argument type is Iterable[int].")
        if not isinstance(volumetric_sizefield_params, VolumetricSizeFieldComputeParams):
            raise TypeError("Invalid argument type passed for volumetric_sizefield_params, valid argument type is VolumetricSizeFieldComputeParams.")
        args = {"size_control_ids" : size_control_ids,
        "volumetric_sizefield_params" : volumetric_sizefield_params._jsonify()}
        command_name = "PrimeMesh::SizeField/ComputeVolumetric"
        self._model._print_logs_before_command("compute_volumetric", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("compute_volumetric", VolumetricSizeFieldComputeResults(model = self._model, json_data = result))
        return VolumetricSizeFieldComputeResults(model = self._model, json_data = result)
